@@ -17,10 +17,11 @@ class SoundSet(Struct):
 			self.sounds.Add(Sound(name))
 
 class Image(Struct):
-	def __init__(self, name="", filename=""):
+	def __init__(self, name="", filename="", linear_mapping=0):
 		Struct.__init__(self, "CDataImage")
 		self.name = String(name)
 		self.filename = String(filename)
+		self.flag = Int(linear_mapping)
 		self.id = TextureHandle()
 
 class SpriteSet(Struct):
@@ -233,16 +234,20 @@ container.sounds.Add(SoundSet("menu", ["audio/music_menu.wv"]))
 image_null = Image("null", "")
 image_particles = Image("particles", "particles.png")
 image_game = Image("game", "game.png")
-image_browseicons = Image("browseicons", "ui/icons/browse.png")
+image_browseicons = Image("browseicons", "ui/icons/browse.png", 1)
 image_emoticons = Image("emoticons", "emoticons.png")
-image_demobuttons = Image("demobuttons", "ui/demo_buttons.png")
-image_fileicons = Image("fileicons", "ui/file_icons.png")
-image_guibuttons = Image("guibuttons", "ui/gui_buttons.png")
-image_guiicons = Image("guiicons", "ui/gui_icons.png")
-image_menuicons = Image("menuicons", "ui/icons/menu.png")
-image_toolicons = Image("toolicons", "ui/icons/tools.png")
-image_friendicons = Image("friendicons", "ui/icons/friend.png")
-image_infoicons = Image("infoicons", "ui/icons/info.png")
+image_demobuttons = Image("demobuttons", "ui/demo_buttons.png", 1)
+image_fileicons = Image("fileicons", "ui/file_icons.png", 1)
+image_guibuttons = Image("guibuttons", "ui/gui_buttons.png", 1)
+image_guiicons = Image("guiicons", "ui/gui_icons.png", 1)
+image_menuicons = Image("menuicons", "ui/icons/menu.png", 1)
+image_toolicons = Image("toolicons", "ui/icons/tools.png", 1)
+image_arrowicons = Image("arrowicons", "ui/icons/arrows.png", 1)
+image_friendicons = Image("friendicons", "ui/icons/friend.png", 1)
+image_infoicons = Image("infoicons", "ui/icons/info.png", 1)
+image_levelicons = Image("levelicons", "ui/icons/level.png", 1)
+image_sidebaricons = Image("sidebaricons", "ui/icons/sidebar.png", 1)
+image_chatwhisper = Image("chatwhisper", "ui/icons/chat_whisper.png", 1)
 
 container.images.Add(image_null)
 container.images.Add(image_game)
@@ -261,8 +266,12 @@ container.images.Add(image_guiicons)
 container.images.Add(Image("no_skinpart", "ui/no_skinpart.png"))
 container.images.Add(image_menuicons)
 container.images.Add(image_toolicons)
+container.images.Add(image_arrowicons)
 container.images.Add(image_friendicons)
 container.images.Add(image_infoicons)
+container.images.Add(image_levelicons)
+container.images.Add(image_sidebaricons)
+container.images.Add(image_chatwhisper)
 
 container.pickups.Add(Pickup("health"))
 container.pickups.Add(Pickup("armor"))
@@ -279,6 +288,7 @@ set_tee_decoration = SpriteSet("tee_decoration", image_null, 2, 1)
 set_tee_hands = SpriteSet("tee_hands", image_null, 2, 1)
 set_tee_feet = SpriteSet("tee_feet", image_null, 2, 1)
 set_tee_eyes = SpriteSet("tee_eyes", image_null, 2, 4)
+set_tee_hats = SpriteSet("tee_hats", image_null, 1, 4)
 set_browseicons = SpriteSet("browseicons", image_browseicons, 4, 2)
 set_emoticons = SpriteSet("emoticons", image_emoticons, 4, 4)
 set_demobuttons = SpriteSet("demobuttons", image_demobuttons, 5, 1)
@@ -287,8 +297,11 @@ set_guibuttons = SpriteSet("guibuttons", image_guibuttons, 12, 4)
 set_guiicons = SpriteSet("guiicons", image_guiicons, 8, 2)
 set_menuicons = SpriteSet("menuicons", image_menuicons, 2, 2)
 set_toolicons = SpriteSet("toolicons", image_toolicons, 4, 2)
+set_arrowicons = SpriteSet("arrowicons", image_arrowicons, 4, 3)
 set_friendicons = SpriteSet("friendicons", image_friendicons, 2, 2)
 set_infoicons = SpriteSet("infoicons", image_infoicons, 1, 2)
+set_levelicons = SpriteSet("levelicons", image_levelicons, 4, 4)
+set_sidebaricons = SpriteSet("sidebaricons", image_sidebaricons, 4, 2)
 
 container.spritesets.Add(set_particles)
 container.spritesets.Add(set_game)
@@ -298,6 +311,7 @@ container.spritesets.Add(set_tee_decoration)
 container.spritesets.Add(set_tee_hands)
 container.spritesets.Add(set_tee_feet)
 container.spritesets.Add(set_tee_eyes)
+container.spritesets.Add(set_tee_hats)
 container.spritesets.Add(set_browseicons)
 container.spritesets.Add(set_emoticons)
 container.spritesets.Add(set_demobuttons)
@@ -306,8 +320,12 @@ container.spritesets.Add(set_guibuttons)
 container.spritesets.Add(set_guiicons)
 container.spritesets.Add(set_menuicons)
 container.spritesets.Add(set_toolicons)
+container.spritesets.Add(set_arrowicons)
 container.spritesets.Add(set_friendicons)
 container.spritesets.Add(set_infoicons)
+container.spritesets.Add(set_levelicons)
+container.spritesets.Add(set_sidebaricons)
+
 
 container.sprites.Add(Sprite("part_slice", set_particles, 0,0,1,1))
 container.sprites.Add(Sprite("part_ball", set_particles, 1,0,1,1))
@@ -387,6 +405,11 @@ container.sprites.Add(Sprite("pickup_ninja", set_game, 2,10,8,2))
 container.sprites.Add(Sprite("flag_blue", set_game, 12,8,4,8))
 container.sprites.Add(Sprite("flag_red", set_game, 16,8,4,8))
 
+container.sprites.Add(Sprite("ninja_bar_full_left", set_game, 21,4,1,2))
+container.sprites.Add(Sprite("ninja_bar_full", set_game, 22,4,1,2))
+container.sprites.Add(Sprite("ninja_bar_empty", set_game, 23,4,1,2))
+container.sprites.Add(Sprite("ninja_bar_empty_right", set_game, 24,4,1,2))
+
 container.sprites.Add(Sprite("tee_body_outline", set_tee_body, 0,0,1,1))
 container.sprites.Add(Sprite("tee_body", set_tee_body, 1,0,1,1))
 container.sprites.Add(Sprite("tee_body_shadow", set_tee_body, 0,1,1,1))
@@ -409,6 +432,11 @@ container.sprites.Add(Sprite("tee_eyes_pain", set_tee_eyes, 0,1,1,1))
 container.sprites.Add(Sprite("tee_eyes_happy", set_tee_eyes, 1,1,1,1))
 container.sprites.Add(Sprite("tee_eyes_surprise", set_tee_eyes, 0,2,1,1))
 
+container.sprites.Add(Sprite("tee_hats_top1", set_tee_hats, 0,0,1,1))
+container.sprites.Add(Sprite("tee_hats_top2", set_tee_hats, 0,1,1,1))
+container.sprites.Add(Sprite("tee_hats_side1", set_tee_hats, 0,2,1,1))
+container.sprites.Add(Sprite("tee_hats_side2", set_tee_hats, 0,3,1,1))
+
 container.sprites.Add(Sprite("oop", set_emoticons, 0, 0, 1, 1))
 container.sprites.Add(Sprite("exclamation", set_emoticons, 1, 0, 1, 1))
 container.sprites.Add(Sprite("hearts", set_emoticons, 2, 0, 1, 1))
@@ -472,6 +500,19 @@ container.sprites.Add(Sprite("tool_edit_b", set_toolicons, 2,1,1,1))
 container.sprites.Add(Sprite("tool_x_a", set_toolicons, 3,0,1,1))
 container.sprites.Add(Sprite("tool_x_b", set_toolicons, 3,1,1,1))
 
+container.sprites.Add(Sprite("arrow_left_a", set_arrowicons, 0,0,1,1))
+container.sprites.Add(Sprite("arrow_left_b", set_arrowicons, 0,1,1,1))
+container.sprites.Add(Sprite("arrow_left_c", set_arrowicons, 0,2,1,1))
+container.sprites.Add(Sprite("arrow_up_a", set_arrowicons, 1,0,1,1))
+container.sprites.Add(Sprite("arrow_up_b", set_arrowicons, 1,1,1,1))
+container.sprites.Add(Sprite("arrow_up_c", set_arrowicons, 1,2,1,1))
+container.sprites.Add(Sprite("arrow_right_a", set_arrowicons, 2,0,1,1))
+container.sprites.Add(Sprite("arrow_right_b", set_arrowicons, 2,1,1,1))
+container.sprites.Add(Sprite("arrow_right_c", set_arrowicons, 2,2,1,1))
+container.sprites.Add(Sprite("arrow_down_a", set_arrowicons, 3,0,1,1))
+container.sprites.Add(Sprite("arrow_down_b", set_arrowicons, 3,1,1,1))
+container.sprites.Add(Sprite("arrow_down_c", set_arrowicons, 3,2,1,1))
+
 container.sprites.Add(Sprite("friend_plus_a", set_friendicons, 0,0,1,1))
 container.sprites.Add(Sprite("friend_plus_b", set_friendicons, 0,1,1,1))
 container.sprites.Add(Sprite("friend_x_a", set_friendicons, 1,0,1,1))
@@ -480,6 +521,24 @@ container.sprites.Add(Sprite("friend_x_b", set_friendicons, 1,1,1,1))
 container.sprites.Add(Sprite("info_a", set_infoicons, 0,0,1,1))
 container.sprites.Add(Sprite("info_b", set_infoicons, 0,1,1,1))
 
+container.sprites.Add(Sprite("level_a_on", set_levelicons, 0,0,1,1))
+container.sprites.Add(Sprite("level_a_a", set_levelicons, 0,1,1,1))
+container.sprites.Add(Sprite("level_a_b", set_levelicons, 0,2,1,1))
+container.sprites.Add(Sprite("level_b_on", set_levelicons, 1,0,1,1))
+container.sprites.Add(Sprite("level_b_a", set_levelicons, 1,1,1,1))
+container.sprites.Add(Sprite("level_b_b", set_levelicons, 1,2,1,1))
+container.sprites.Add(Sprite("level_c_on", set_levelicons, 2,0,1,1))
+container.sprites.Add(Sprite("level_c_a", set_levelicons, 2,1,1,1))
+container.sprites.Add(Sprite("level_c_b", set_levelicons, 2,2,1,1))
+
+container.sprites.Add(Sprite("sidebar_refresh_a", set_sidebaricons, 0,0,1,1))
+container.sprites.Add(Sprite("sidebar_refresh_b", set_sidebaricons, 0,1,1,1))
+container.sprites.Add(Sprite("sidebar_friend_a", set_sidebaricons, 1,0,1,1))
+container.sprites.Add(Sprite("sidebar_friend_b", set_sidebaricons, 1,1,1,1))
+container.sprites.Add(Sprite("sidebar_filter_a", set_sidebaricons, 2,0,1,1))
+container.sprites.Add(Sprite("sidebar_filter_b", set_sidebaricons, 2,1,1,1))
+container.sprites.Add(Sprite("sidebar_info_a", set_sidebaricons, 3,0,1,1))
+container.sprites.Add(Sprite("sidebar_info_b", set_sidebaricons, 3,1,1,1))
 
 anim = Animation("base")
 anim.body.frames.Add(AnimKeyframe(0, 0, -4, 0))
@@ -552,7 +611,7 @@ weapon.damage.Set(1)
 weapon.ammoregentime.Set(500)
 weapon.visual_size.Set(64)
 weapon.offsetx.Set(32)
-weapon.offsety.Set(4)
+weapon.offsety.Set(-4)
 weapon.muzzleoffsetx.Set(50)
 weapon.muzzleoffsety.Set(6)
 container.weapons.gun.base.Set(weapon)
