@@ -360,6 +360,8 @@ IGraphics::CTextureHandle CGraphics_Threaded::LoadTextureRaw(int Width, int Heig
 	}
 	if(Flags&IGraphics::TEXLOAD_MULTI_DIMENSION)
 		Cmd.m_Flags |= CCommandBuffer::TEXFLAG_TEXTURE3D;
+	if(Flags&IGraphics::TEXLOAD_LINEARMIPMAPS)
+		Cmd.m_Flags |= CCommandBuffer::TEXTFLAG_LINEARMIPMAPS;
 
 	
 	// copy texture data
@@ -520,6 +522,7 @@ void CGraphics_Threaded::QuadsBegin()
 	QuadsSetSubset(0,0,1,1,-1);
 	QuadsSetRotation(0);
 	SetColor(1,1,1,1);
+	m_TextureArrayIndex = m_pBackend->GetTextureArraySize() > 1 ? -1 : 0;
 }
 
 void CGraphics_Threaded::QuadsEnd()
@@ -570,17 +573,36 @@ void CGraphics_Threaded::SetColor4(vec4 TopLeft, vec4 TopRight, vec4 BottomLeft,
 	SetColorVertex(Array, 4);
 }
 
+void CGraphics_Threaded::TilesetFallbackSystem(int TextureIndex)
+{
+	int NewTextureArrayIndex = TextureIndex / (256 / m_pBackend->GetTextureArraySize());
+	if(m_TextureArrayIndex == -1)
+		m_TextureArrayIndex = NewTextureArrayIndex;
+	else if(m_TextureArrayIndex != NewTextureArrayIndex)
+	{
+		// have to switch the texture index
+		FlushVertices();
+		m_TextureArrayIndex = NewTextureArrayIndex;
+	}
+}
+
 void CGraphics_Threaded::QuadsSetSubset(float TlU, float TlV, float BrU, float BrV, int TextureIndex)
 {
 	dbg_assert(m_Drawing == DRAWING_QUADS, "called Graphics()->QuadsSetSubset without begin");
 
+	// tileset fallback system
+	if(m_pBackend->GetTextureArraySize() > 1 && TextureIndex >= 0)
+		TilesetFallbackSystem(TextureIndex);
+
+	m_State.m_TextureArrayIndex = m_TextureArrayIndex;
+
 	m_aTexture[0].u = TlU;	m_aTexture[1].u = BrU;
 	m_aTexture[0].v = TlV;	m_aTexture[1].v = TlV;
 
 	m_aTexture[3].u = TlU;	m_aTexture[2].u = BrU;
 	m_aTexture[3].v = BrV;	m_aTexture[2].v = BrV;
 
-	m_aTexture[0].i = m_aTexture[1].i = m_aTexture[2].i = m_aTexture[3].i = (0.5f + TextureIndex) / 256.0f;
+	m_aTexture[0].i = m_aTexture[1].i = m_aTexture[2].i = m_aTexture[3].i = (0.5f + TextureIndex) / (256.0f/m_pBackend->GetTextureArraySize());
 	m_State.m_Dimension = (TextureIndex < 0) ? 2 : 3;
 }
 
@@ -588,12 +610,18 @@ void CGraphics_Threaded::QuadsSetSubsetFree(
 	float x0, float y0, float x1, float y1,
 	float x2, float y2, float x3, float y3, int TextureIndex)
 {
+	// tileset fallback system
+	if(m_pBackend->GetTextureArraySize() > 1 && TextureIndex >= 0)
+		TilesetFallbackSystem(TextureIndex);
+	
+	m_State.m_TextureArrayIndex = m_TextureArrayIndex;
+
 	m_aTexture[0].u = x0; m_aTexture[0].v = y0;
 	m_aTexture[1].u = x1; m_aTexture[1].v = y1;
 	m_aTexture[2].u = x2; m_aTexture[2].v = y2;
 	m_aTexture[3].u = x3; m_aTexture[3].v = y3;
 
-	m_aTexture[0].i = m_aTexture[1].i = m_aTexture[2].i = m_aTexture[3].i = (0.5f + TextureIndex) / 256.0f;
+	m_aTexture[0].i = m_aTexture[1].i = m_aTexture[2].i = m_aTexture[3].i = (0.5f + TextureIndex) / (256.0f/m_pBackend->GetTextureArraySize());
 	m_State.m_Dimension = (TextureIndex < 0) ? 2 : 3;
 }
 
