@@ -67,6 +67,11 @@ public:
 	CFontSizeData m_aSizes[NUM_FONT_SIZES];
 };
 
+struct CQuadChar
+{
+	float m_aUvs[4];
+	IGraphics::CQuadItem m_QuadItem;
+};
 
 class CTextRender : public IEngineTextRender
 {
@@ -165,7 +170,7 @@ class CTextRender : public IEngineTextRender
 		pSizeData->m_TextureWidth = Width;
 		pSizeData->m_TextureHeight = Height;
 		pSizeData->m_CurrentCharacter = 0;
-		
+
 		dbg_msg("", "pFont memory usage: %d", FontMemoryUsage);
 
 		mem_free(pMem);
@@ -493,6 +498,7 @@ public:
 		pCursor->m_LineCount = 1;
 		pCursor->m_LineWidth = -1;
 		pCursor->m_Flags = Flags;
+		pCursor->m_GlyphCount = 0;
 		pCursor->m_CharCount = 0;
 	}
 
@@ -538,6 +544,228 @@ public:
 		m_TextOutlineA = a;
 	}
 
+	virtual void TextShadowed(CTextCursor *pCursor, const char *pText, int Length, vec2 ShadowOffset,
+							  vec4 ShadowColor, vec4 TextColor_)
+	{
+		float ScreenX0, ScreenY0, ScreenX1, ScreenY1;
+		float FakeToScreenX, FakeToScreenY;
+
+		// to correct coords, convert to screen coords, round, and convert back
+		Graphics()->GetScreen(&ScreenX0, &ScreenY0, &ScreenX1, &ScreenY1);
+
+		FakeToScreenX = (Graphics()->ScreenWidth()/(ScreenX1-ScreenX0));
+		FakeToScreenY = (Graphics()->ScreenHeight()/(ScreenY1-ScreenY0));
+		ShadowOffset.x /= FakeToScreenX;
+		ShadowOffset.y /= FakeToScreenY;
+
+		CQuadChar aTextQuads[1024];
+		int TextQuadCount = 0;
+		IGraphics::CTextureHandle FontTexture;
+
+		TextDeferredRenderEx(pCursor, pText, Length, aTextQuads, sizeof(aTextQuads)/sizeof(aTextQuads[0]),
+				&TextQuadCount, &FontTexture);
+
+		Graphics()->TextureSet(FontTexture);
+
+		Graphics()->QuadsBegin();
+
+		// shadow pass
+		Graphics()->SetColor(ShadowColor.r, ShadowColor.g, ShadowColor.b, ShadowColor.a);
+
+		for(int i = 0; i < TextQuadCount; i++)
+		{
+			const CQuadChar& q = aTextQuads[i];
+			Graphics()->QuadsSetSubset(q.m_aUvs[0], q.m_aUvs[1], q.m_aUvs[2], q.m_aUvs[3]);
+
+			IGraphics::CQuadItem QuadItem = q.m_QuadItem;
+			QuadItem.m_X += ShadowOffset.x;
+			QuadItem.m_Y += ShadowOffset.y;
+			Graphics()->QuadsDrawTL(&QuadItem, 1);
+		}
+
+		// text pass
+		Graphics()->SetColor(TextColor_.r, TextColor_.g, TextColor_.b, TextColor_.a);
+
+		for(int i = 0; i < TextQuadCount; i++)
+		{
+			const CQuadChar& q = aTextQuads[i];
+			Graphics()->QuadsSetSubset(q.m_aUvs[0], q.m_aUvs[1], q.m_aUvs[2], q.m_aUvs[3]);
+			Graphics()->QuadsDrawTL(&q.m_QuadItem, 1);
+		}
+
+		Graphics()->QuadsEnd();
+	}
+
+	virtual void TextDeferredRenderEx(CTextCursor *pCursor, const char *pText, int Length,
+									  CQuadChar* aQuadChar, int QuadCharMaxCount, int* pQuadCharCount,
+									  IGraphics::CTextureHandle* pFontTexture)
+	{
+		CFont *pFont = pCursor->m_pFont;
+		CFontSizeData *pSizeData = NULL;
+
+		float ScreenX0, ScreenY0, ScreenX1, ScreenY1;
+		float FakeToScreenX, FakeToScreenY;
+		int ActualX, ActualY;
+
+		int ActualSize;
+		int GotNewLine = 0;
+		float DrawX = 0.0f, DrawY = 0.0f;
+		int LineCount = 0;
+		float CursorX, CursorY;
+
+		float Size = pCursor->m_FontSize;
+
+		// to correct coords, convert to screen coords, round, and convert back
+		Graphics()->GetScreen(&ScreenX0, &ScreenY0, &ScreenX1, &ScreenY1);
+
+		FakeToScreenX = (Graphics()->ScreenWidth()/(ScreenX1-ScreenX0));
+		FakeToScreenY = (Graphics()->ScreenHeight()/(ScreenY1-ScreenY0));
+		ActualX = (int)(pCursor->m_X * FakeToScreenX);
+		ActualY = (int)(pCursor->m_Y * FakeToScreenY);
+
+		CursorX = ActualX / FakeToScreenX;
+		CursorY = ActualY / FakeToScreenY;
+
+		// same with size
+		ActualSize = (int)(Size * FakeToScreenY);
+		Size = ActualSize / FakeToScreenY;
+
+		// fetch pFont data
+		if(!pFont)
+			pFont = m_pDefaultFont;
+
+		if(!pFont)
+			return;
+
+		pSizeData = GetSize(pFont, ActualSize);
+		RenderSetup(pFont, ActualSize);
+		*pFontTexture = pSizeData->m_aTextures[0];
+
+		float Scale = 1.0f/pSizeData->m_FontSize;
+
+		// set length
+		if(Length < 0)
+			Length = str_length(pText);
+
+
+		const char *pCurrent = (char *)pText;
+		const char *pEnd = pCurrent+Length;
+		DrawX = CursorX;
+		DrawY = CursorY;
+		LineCount = pCursor->m_LineCount;
+
+		while(pCurrent < pEnd && (pCursor->m_MaxLines < 1 || LineCount <= pCursor->m_MaxLines))
+		{
+			int NewLine = 0;
+			const char *pBatchEnd = pEnd;
+			if(pCursor->m_LineWidth > 0 && !(pCursor->m_Flags&TEXTFLAG_STOP_AT_END))
+			{
+				int Wlen = min(WordLength((char *)pCurrent), (int)(pEnd-pCurrent));
+				CTextCursor Compare = *pCursor;
+				Compare.m_X = DrawX;
+				Compare.m_Y = DrawY;
+				Compare.m_Flags &= ~TEXTFLAG_RENDER;
+				Compare.m_LineWidth = -1;
+				TextDeferredRenderEx(&Compare, pCurrent, Wlen, aQuadChar, QuadCharMaxCount, pQuadCharCount,
+									 pFontTexture);
+
+				if(Compare.m_X-DrawX > pCursor->m_LineWidth)
+				{
+					// word can't be fitted in one line, cut it
+					CTextCursor Cutter = *pCursor;
+					Cutter.m_GlyphCount = 0;
+					Cutter.m_X = DrawX;
+					Cutter.m_Y = DrawY;
+					Cutter.m_Flags &= ~TEXTFLAG_RENDER;
+					Cutter.m_Flags |= TEXTFLAG_STOP_AT_END;
+
+					TextDeferredRenderEx(&Cutter, (const char *)pCurrent, Wlen, aQuadChar, QuadCharMaxCount,
+										 pQuadCharCount, pFontTexture);
+					Wlen = Cutter.m_GlyphCount;
+					NewLine = 1;
+
+					if(Wlen <= 3) // if we can't place 3 chars of the word on this line, take the next
+						Wlen = 0;
+				}
+				else if(Compare.m_X-pCursor->m_StartX > pCursor->m_LineWidth)
+				{
+					NewLine = 1;
+					Wlen = 0;
+				}
+
+				pBatchEnd = pCurrent + Wlen;
+			}
+
+			const char *pTmp = pCurrent;
+			int NextCharacter = str_utf8_decode(&pTmp);
+			while(pCurrent < pBatchEnd)
+			{
+				pCursor->m_CharCount += pTmp-pCurrent;
+				int Character = NextCharacter;
+				pCurrent = pTmp;
+				NextCharacter = str_utf8_decode(&pTmp);
+
+				if(Character == '\n')
+				{
+					DrawX = pCursor->m_StartX;
+					DrawY += Size;
+					DrawX = (int)(DrawX * FakeToScreenX) / FakeToScreenX; // realign
+					DrawY = (int)(DrawY * FakeToScreenY) / FakeToScreenY;
+					++LineCount;
+					if(pCursor->m_MaxLines > 0 && LineCount > pCursor->m_MaxLines)
+						break;
+					continue;
+				}
+
+				CFontChar *pChr = GetChar(pFont, pSizeData, Character);
+				if(pChr)
+				{
+					float Advance = pChr->m_AdvanceX + Kerning(pFont, Character, NextCharacter)*Scale;
+					if(pCursor->m_Flags&TEXTFLAG_STOP_AT_END && DrawX+Advance*Size-pCursor->m_StartX > pCursor->m_LineWidth)
+					{
+						// we hit the end of the line, no more to render or count
+						pCurrent = pEnd;
+						break;
+					}
+
+					if(pCursor->m_Flags&TEXTFLAG_RENDER)
+					{
+						dbg_assert(*pQuadCharCount < QuadCharMaxCount, "aQuadChar size is too small");
+
+						CQuadChar QuadChar;
+						memmove(QuadChar.m_aUvs, pChr->m_aUvs, sizeof(pChr->m_aUvs));
+
+						IGraphics::CQuadItem QuadItem(DrawX+pChr->m_OffsetX*Size,
+													  DrawY+pChr->m_OffsetY*Size,
+													  pChr->m_Width*Size,
+													  pChr->m_Height*Size);
+						QuadChar.m_QuadItem = QuadItem;
+						aQuadChar[(*pQuadCharCount)++] = QuadChar;
+					}
+
+					DrawX += Advance*Size;
+					pCursor->m_GlyphCount++;
+				}
+			}
+
+			if(NewLine)
+			{
+				DrawX = pCursor->m_StartX;
+				DrawY += Size;
+				GotNewLine = 1;
+				DrawX = (int)(DrawX * FakeToScreenX) / FakeToScreenX; // realign
+				DrawY = (int)(DrawY * FakeToScreenY) / FakeToScreenY;
+				++LineCount;
+			}
+		}
+
+		pCursor->m_X = DrawX;
+		pCursor->m_LineCount = LineCount;
+
+		if(GotNewLine)
+			pCursor->m_Y = DrawY;
+	}
+
 	virtual void TextEx(CTextCursor *pCursor, const char *pText, int Length)
 	{
 		CFont *pFont = pCursor->m_pFont;
@@ -635,14 +863,14 @@ public:
 					{
 						// word can't be fitted in one line, cut it
 						CTextCursor Cutter = *pCursor;
-						Cutter.m_CharCount = 0;
+						Cutter.m_GlyphCount = 0;
 						Cutter.m_X = DrawX;
 						Cutter.m_Y = DrawY;
 						Cutter.m_Flags &= ~TEXTFLAG_RENDER;
 						Cutter.m_Flags |= TEXTFLAG_STOP_AT_END;
 
 						TextEx(&Cutter, (const char *)pCurrent, Wlen);
-						Wlen = Cutter.m_CharCount;
+						Wlen = Cutter.m_GlyphCount;
 						NewLine = 1;
 
 						if(Wlen <= 3) // if we can't place 3 chars of the word on this line, take the next
@@ -661,6 +889,7 @@ public:
 				int NextCharacter = str_utf8_decode(&pTmp);
 				while(pCurrent < pBatchEnd)
 				{
+					pCursor->m_CharCount += pTmp-pCurrent;
 					int Character = NextCharacter;
 					pCurrent = pTmp;
 					NextCharacter = str_utf8_decode(&pTmp);
@@ -696,7 +925,7 @@ public:
 						}
 
 						DrawX += Advance*Size;
-						pCursor->m_CharCount++;
+						pCursor->m_GlyphCount++;
 					}
 				}
 
@@ -722,6 +951,38 @@ public:
 			pCursor->m_Y = DrawY;
 	}
 
+	float TextGetLineBaseY(const CTextCursor *pCursor)
+	{
+		CFont *pFont = pCursor->m_pFont;
+		CFontSizeData *pSizeData = NULL;
+
+		float ScreenX0, ScreenY0, ScreenX1, ScreenY1;
+		float Size = pCursor->m_FontSize;
+
+		// to correct coords, convert to screen coords, round, and convert back
+		Graphics()->GetScreen(&ScreenX0, &ScreenY0, &ScreenX1, &ScreenY1);
+
+		float FakeToScreenY = (Graphics()->ScreenHeight()/(ScreenY1-ScreenY0));
+		int ActualY = (int)(pCursor->m_Y * FakeToScreenY);
+		float CursorY = ActualY / FakeToScreenY;
+
+		// same with size
+		int ActualSize = (int)(Size * FakeToScreenY);
+		Size = ActualSize / FakeToScreenY;
+
+		// fetch pFont data
+		if(!pFont)
+			pFont = m_pDefaultFont;
+
+		if(!pFont)
+			return 0;
+
+		pSizeData = GetSize(pFont, ActualSize);
+		RenderSetup(pFont, ActualSize);
+		CFontChar *pChr = GetChar(pFont, pSizeData, ' ');
+		return CursorY + pChr->m_OffsetY*Size + pChr->m_Height*Size;
+	}
+
 };
 
 IEngineTextRender *CreateEngineTextRender() { return new CTextRender; }
