@@ -608,3 +608,7 @@ void CNetBan::ConBansSave(IConsole::IResult *pResult, void *pUser)
 	str_format(aBuf, sizeof(aBuf), "saved banlist to '%s'", pFilename);
 	pThis->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "net_ban", aBuf);
 }
+
+// explicitly instantiate template for src/engine/server/server.cpp
+template void CNetBan::MakeBanInfo<CNetRange>(const CBan<CNetRange> *pBan, char *pBuf, unsigned BufferSize, int Type) const;
+template void CNetBan::MakeBanInfo<NETADDR>(const CBan<NETADDR> *pBan, char *pBuf, unsigned BufferSize, int Type) const;
