@@ -81,7 +81,7 @@ void CChat::ConSayTeam(IConsole::IResult *pResult, void *pUserData)
 void CChat::ConWhisper(IConsole::IResult *pResult, void *pUserData)
 {
 	CChat *pChat = (CChat *)pUserData;
-	
+
 	int Target = pResult->GetInteger(0);
 	if(Target < 0 || Target >= MAX_CLIENTS || !pChat->m_pClient->m_aClients[Target].m_Active || pChat->m_pClient->m_LocalClientID == Target)
 		pChat->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", "please enter a valid ClientID");
@@ -103,15 +103,15 @@ void CChat::ConChat(IConsole::IResult *pResult, void *pUserData)
 		pChat->EnableMode(CHAT_TEAM);
 	else if(str_comp(pMode, "whisper") == 0)
 	{
-		int Target = -1;
+		int Target = pChat->m_WhisperTarget; // default to ID of last target
 		if(pResult->NumArguments() == 2)
 			Target = pResult->GetInteger(1);
 		else
 		{
-			// pick next player as target
+			// pick next valid player as target
 			for(int i = 0; i < MAX_CLIENTS; i++)
 			{
-				int ClientID = (pChat->m_pClient->m_LocalClientID + i) % MAX_CLIENTS;
+				int ClientID = (Target + i) % MAX_CLIENTS;
 				if(pChat->m_pClient->m_aClients[ClientID].m_Active && pChat->m_pClient->m_LocalClientID != ClientID)
 				{
 					Target = ClientID;
@@ -120,7 +120,10 @@ void CChat::ConChat(IConsole::IResult *pResult, void *pUserData)
 			}
 		}
 		if(Target < 0 || Target >= MAX_CLIENTS || !pChat->m_pClient->m_aClients[Target].m_Active || pChat->m_pClient->m_LocalClientID == Target)
-			pChat->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", "please enter a valid ClientID");
+		{
+			if(pResult->NumArguments() == 2)
+				pChat->Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, "console", "please enter a valid ClientID");
+		}
 		else
 		{
 			pChat->m_WhisperTarget = Target;
@@ -161,7 +164,7 @@ bool CChat::OnInput(IInput::CEvent Event)
 		{
 			bool AddEntry = false;
 
-			if(m_LastChatSend+time_freq() < time_get())
+			if(m_PendingChatCounter == 0 && m_LastChatSend+time_freq() < time_get())
 			{
 				Say(m_Mode, m_Input.GetString());
 				AddEntry = true;
@@ -187,10 +190,14 @@ bool CChat::OnInput(IInput::CEvent Event)
 	{
 		if (m_Mode == CHAT_WHISPER)
 		{
-			// pick next player as target
+			// change target
 			for(int i = 0; i < MAX_CLIENTS; i++)
 			{
-				int ClientID = (m_WhisperTarget + i) % MAX_CLIENTS;
+				int ClientID;
+				if(Input()->KeyIsPressed(KEY_LCTRL) || Input()->KeyIsPressed(KEY_RCTRL))
+					ClientID = (m_WhisperTarget + MAX_CLIENTS - i) % MAX_CLIENTS; // pick previous player as target
+				else
+					ClientID = (m_WhisperTarget + i) % MAX_CLIENTS; // pick next player as target
 				if (m_pClient->m_aClients[ClientID].m_Active && m_WhisperTarget != ClientID && m_pClient->m_LocalClientID != ClientID)
 				{
 					m_WhisperTarget = ClientID;
@@ -299,7 +306,7 @@ bool CChat::OnInput(IInput::CEvent Event)
 		if(m_Input.ProcessInput(Event))
 		{
 			m_InputUpdate = true;
-			
+
 			// reset name completion process
 			m_CompletionChosen = -1;
 		}
@@ -359,15 +366,16 @@ void CChat::OnMessage(int MsgType, void *pRawMsg)
 	if(MsgType == NETMSGTYPE_SV_CHAT)
 	{
 		CNetMsg_Sv_Chat *pMsg = (CNetMsg_Sv_Chat *)pRawMsg;
-		AddLine(pMsg->m_ClientID, pMsg->m_Mode, pMsg->m_pMessage);
+		AddLine(pMsg->m_ClientID, pMsg->m_Mode, pMsg->m_pMessage, pMsg->m_TargetID);
 	}
 }
 
-void CChat::AddLine(int ClientID, int Mode, const char *pLine)
+void CChat::AddLine(int ClientID, int Mode, const char *pLine, int TargetID)
 {
-	if(*pLine == 0 || (ClientID != -1 && (!g_Config.m_ClShowsocial || m_pClient->m_aClients[ClientID].m_aName[0] == '\0' || // unknown client
+	if(*pLine == 0 || (ClientID != -1 && (!g_Config.m_ClShowsocial || !m_pClient->m_aClients[ClientID].m_Active || // unknown client
 		m_pClient->m_aClients[ClientID].m_ChatIgnore ||
-		(m_pClient->m_LocalClientID != ClientID && g_Config.m_ClShowChatFriends && !m_pClient->m_aClients[ClientID].m_Friend))))
+		g_Config.m_ClFilterchat == 2 ||
+		(m_pClient->m_LocalClientID != ClientID && g_Config.m_ClFilterchat == 1 && !m_pClient->m_aClients[ClientID].m_Friend))))
 		return;
 
 	// trim right and set maximum length to 128 utf8-characters
@@ -375,7 +383,7 @@ void CChat::AddLine(int ClientID, int Mode, const char *pLine)
 	const char *pStr = pLine;
 	const char *pEnd = 0;
 	while(*pStr)
- 	{
+	{
 		const char *pStrOld = pStr;
 		int Code = str_utf8_decode(&pStr);
 
@@ -394,7 +402,7 @@ void CChat::AddLine(int ClientID, int Mode, const char *pLine)
 			*(const_cast<char *>(pStr)) = 0;
 			break;
 		}
- 	}
+	}
 	if(pEnd != 0)
 		*(const_cast<char *>(pEnd)) = 0;
 
@@ -402,7 +410,6 @@ void CChat::AddLine(int ClientID, int Mode, const char *pLine)
 	char *p = const_cast<char*>(pLine);
 	while(*p)
 	{
-		Highlighted = false;
 		pLine = p;
 		// find line seperator and strip multiline
 		while(*p)
@@ -416,28 +423,43 @@ void CChat::AddLine(int ClientID, int Mode, const char *pLine)
 
 		m_CurrentLine = (m_CurrentLine+1)%MAX_LINES;
 		m_aLines[m_CurrentLine].m_Time = time_get();
-		m_aLines[m_CurrentLine].m_YOffset[0] = -1.0f;
-		m_aLines[m_CurrentLine].m_YOffset[1] = -1.0f;
+		m_aLines[m_CurrentLine].m_Size[0].y = -1.0f;
+		m_aLines[m_CurrentLine].m_Size[1].y = -1.0f;
 		m_aLines[m_CurrentLine].m_ClientID = ClientID;
+		m_aLines[m_CurrentLine].m_TargetID = TargetID;
 		m_aLines[m_CurrentLine].m_Mode = Mode;
 		m_aLines[m_CurrentLine].m_NameColor = -2;
 
 		// check for highlighted name
-		const char *pHL = str_find_nocase(pLine, m_pClient->m_aClients[m_pClient->m_LocalClientID].m_aName);
-		if(pHL)
+		Highlighted = false;
+		// do not highlight our own messages, whispers and system messages
+		if(Mode != CHAT_WHISPER && ClientID != -1 && ClientID != m_pClient->m_LocalClientID)
 		{
-			int Length = str_length(m_pClient->m_aClients[m_pClient->m_LocalClientID].m_aName);
-			if((pLine == pHL || pHL[-1] == ' ') && (pHL[Length] == 0 || pHL[Length] == ' ' || (pHL[Length] == ':' && pHL[Length+1] == ' ')))
-				Highlighted = true;
-			m_CompletionFav = ClientID;
+			const char *pHL = str_find_nocase(pLine, m_pClient->m_aClients[m_pClient->m_LocalClientID].m_aName);
+			if(pHL)
+			{
+				int Length = str_length(m_pClient->m_aClients[m_pClient->m_LocalClientID].m_aName);
+				if((pLine == pHL || pHL[-1] == ' ')) // "" or " " before
+				{
+					if((pHL[Length] == 0 || pHL[Length] == ' ')) // "" or " " after
+						Highlighted = true;
+					if(pHL[Length] == ':' && (pHL[Length+1] == 0 || pHL[Length+1] == ' ')) // ":" or ": " after
+						Highlighted = true;
+				}
+				m_CompletionFav = ClientID;
+			}
 		}
 
-		m_aLines[m_CurrentLine].m_Highlighted =  Highlighted || Mode == CHAT_WHISPER;
+		m_aLines[m_CurrentLine].m_Highlighted =  Highlighted;
+
+		int NameCID = ClientID;
+		if(Mode == CHAT_WHISPER && ClientID == m_pClient->m_LocalClientID && TargetID >= 0)
+			NameCID = TargetID;
 
 		if(ClientID == -1) // server message
 		{
-			str_copy(m_aLines[m_CurrentLine].m_aName, "*** ", sizeof(m_aLines[m_CurrentLine].m_aName));
-			str_format(m_aLines[m_CurrentLine].m_aText, sizeof(m_aLines[m_CurrentLine].m_aText), "%s", pLine);
+			m_aLines[m_CurrentLine].m_aName[0] = 0;
+			str_format(m_aLines[m_CurrentLine].m_aText, sizeof(m_aLines[m_CurrentLine].m_aText), "*** %s", pLine);
 		}
 		else
 		{
@@ -452,8 +474,8 @@ void CChat::AddLine(int ClientID, int Mode, const char *pLine)
 					m_aLines[m_CurrentLine].m_NameColor = TEAM_BLUE;
 			}
 
-			str_format(m_aLines[m_CurrentLine].m_aName, sizeof(m_aLines[m_CurrentLine].m_aName), "%2d: %s", ClientID, m_pClient->m_aClients[ClientID].m_aName);
-			str_format(m_aLines[m_CurrentLine].m_aText, sizeof(m_aLines[m_CurrentLine].m_aText), ": %s", pLine);
+			str_format(m_aLines[m_CurrentLine].m_aName, sizeof(m_aLines[m_CurrentLine].m_aName), "%s", m_pClient->m_aClients[NameCID].m_aName);
+			str_format(m_aLines[m_CurrentLine].m_aText, sizeof(m_aLines[m_CurrentLine].m_aText), "%s", pLine);
 		}
 
 		char aBuf[1024];
@@ -464,8 +486,9 @@ void CChat::AddLine(int ClientID, int Mode, const char *pLine)
 			str_copy(aBufMode, "teamchat", sizeof(aBufMode));
 		else
 			str_copy(aBufMode, "chat", sizeof(aBufMode));
-		str_format(aBuf, sizeof(aBuf), "%s%s", m_aLines[m_CurrentLine].m_aName, m_aLines[m_CurrentLine].m_aText);
-		Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, aBufMode, aBuf, Highlighted);
+
+		str_format(aBuf, sizeof(aBuf), "%2d: %s: %s", NameCID, m_aLines[m_CurrentLine].m_aName, m_aLines[m_CurrentLine].m_aText);
+		Console()->Print(IConsole::OUTPUT_LEVEL_STANDARD, aBufMode, aBuf, Highlighted || Mode == CHAT_WHISPER);
 	}
 
 	// play sound
@@ -478,7 +501,7 @@ void CChat::AddLine(int ClientID, int Mode, const char *pLine)
 			m_aLastSoundPlayed[CHAT_SERVER] = Now;
 		}
 	}
-	else if(Highlighted)
+	else if(Highlighted || Mode == CHAT_WHISPER)
 	{
 		if(Now-m_aLastSoundPlayed[CHAT_HIGHLIGHT] >= time_freq()*3/10)
 		{
@@ -517,30 +540,112 @@ void CChat::OnRender()
 	if(m_pClient->m_pMenus->IsActive())
 		return;
 
-	float Width = 300.0f*Graphics()->ScreenAspect();
-	Graphics()->MapScreen(0.0f, 0.0f, Width, 300.0f);
-	float x = 5.0f;
-	float y = 300.0f-20.0f;
-	if(m_Mode != CHAT_NONE)
+	const float Height = 300.0f;
+	const float Width = Height*Graphics()->ScreenAspect();
+	Graphics()->MapScreen(0.0f, 0.0f, Width, Height);
+	float x = 12.0f;
+	float y = Height-20.0f;
+	const int LocalCID = m_pClient->m_LocalClientID;
+	const CGameClient::CClientData& LocalClient = m_pClient->m_aClients[LocalCID];
+	const int LocalTteam = LocalClient.m_Team;
+
+	if(m_Mode == CHAT_WHISPER && !m_pClient->m_aClients[m_WhisperTarget].m_Active)
+		m_Mode = CHAT_NONE;
+	else if(m_Mode != CHAT_NONE)
 	{
+		// calculate category text size
+		// TODO: rework TextRender. Writing the same code twice to calculate a simple thing as width is ridiculus
+		float CategoryWidth = 0;
+		float CategoryHeight;
+		const float CategoryFontSize = 8.0f;
+		const float InputFontSize = 8.0f;
+		char aCatText[48];
+
+		{
+			CTextCursor Cursor;
+			TextRender()->SetCursor(&Cursor, x, y, CategoryFontSize, 0);
+
+			if(m_Mode == CHAT_ALL)
+				str_copy(aCatText, Localize("All"), sizeof(aCatText));
+			else if(m_Mode == CHAT_TEAM)
+			{
+				if(LocalTteam == TEAM_SPECTATORS)
+					str_copy(aCatText, Localize("Spectators"), sizeof(aCatText));
+				else
+					str_copy(aCatText, Localize("Team"), sizeof(aCatText));
+			}
+			else if(m_Mode == CHAT_WHISPER)
+			{
+				CategoryWidth += RenderTools()->GetClientIdRectSize(CategoryFontSize);
+				str_format(aCatText, sizeof(aCatText), "%s",m_pClient->m_aClients[m_WhisperTarget].m_aName);
+			}
+			else
+				str_copy(aCatText, Localize("Chat"), sizeof(aCatText));
+
+			TextRender()->TextEx(&Cursor, aCatText, -1);
+
+			CategoryWidth += Cursor.m_X - Cursor.m_StartX;
+			CategoryHeight = Cursor.m_FontSize;
+		}
+
+		// draw a background box
+		const vec4 CRCWhite(1, 1, 1, 0.25);
+		const vec4 CRCTeam(0.4, 1, 0.4, 0.4);
+		const vec4 CRCWhisper(0, 0.5, 1, 0.5);
+
+		vec4 CatRectColor = CRCWhite;
+		if(m_Mode == CHAT_TEAM)
+			CatRectColor = CRCTeam;
+		else if(m_Mode == CHAT_WHISPER)
+			CatRectColor = CRCWhisper;
+
+		const float IconOffsetX = m_Mode == CHAT_WHISPER ? 6.0f : 0.0f;
+
+		CUIRect CatRect;
+		CatRect.x = 0;
+		CatRect.y = y;
+		CatRect.w = CategoryWidth + x + 2.0f + IconOffsetX;
+		CatRect.h = CategoryHeight + 4.0f;
+		RenderTools()->DrawUIRect(&CatRect, CatRectColor, CUI::CORNER_R, 2.0f);
+
+		// draw chat icon
+		Graphics()->WrapClamp();
+		IGraphics::CQuadItem QuadIcon;
+
+		if(m_Mode == CHAT_WHISPER)
+		{
+			Graphics()->TextureSet(g_pData->m_aImages[IMAGE_CHATWHISPER].m_Id);
+			Graphics()->QuadsBegin();
+			Graphics()->QuadsSetSubset(1, 0, 0, 1);
+			QuadIcon = IGraphics::CQuadItem(1.5f, y + (CatRect.h - 8.0f) * 0.5f, 16.f, 8.0f);
+		}
+		else
+		{
+			Graphics()->TextureSet(g_pData->m_aImages[IMAGE_EMOTICONS].m_Id);
+			Graphics()->QuadsBegin();
+			RenderTools()->SelectSprite(SPRITE_DOTDOT);
+			QuadIcon = IGraphics::CQuadItem(1.0f, y, 10.f, 10.0f);
+		}
+
+		Graphics()->SetColor(1, 1, 1, 1);
+		Graphics()->QuadsDrawTL(&QuadIcon, 1);
+		Graphics()->QuadsEnd();
+		Graphics()->WrapNormal();
+
 		// render chat input
 		CTextCursor Cursor;
-		TextRender()->SetCursor(&Cursor, x, y, 8.0f, TEXTFLAG_RENDER);
+		TextRender()->SetCursor(&Cursor, x + IconOffsetX, y, CategoryFontSize, TEXTFLAG_RENDER);
 		Cursor.m_LineWidth = Width-190.0f;
 		Cursor.m_MaxLines = 2;
 
-		char aBuf[32];
-		if(m_Mode == CHAT_ALL)
-			str_copy(aBuf, Localize("All"), sizeof(aBuf));
-		else if(m_Mode == CHAT_TEAM)
-			str_copy(aBuf, Localize("Team"), sizeof(aBuf));
-		else if(m_Mode == CHAT_WHISPER)
-			str_format(aBuf, sizeof(aBuf), "%s %2d: %s", Localize("To"), m_WhisperTarget, m_pClient->m_aClients[m_WhisperTarget].m_aName);
-		else
-			str_copy(aBuf, Localize("Chat"), sizeof(aBuf));
-			
-		TextRender()->TextEx(&Cursor, aBuf, -1);
-		TextRender()->TextEx(&Cursor, ": ", -1);
+		if(m_Mode == CHAT_WHISPER)
+			RenderTools()->DrawClientID(TextRender(), &Cursor, m_WhisperTarget);
+		TextRender()->TextEx(&Cursor, aCatText, -1);
+
+		Cursor.m_X += 4.0f;
+		Cursor.m_Y -= (InputFontSize-CategoryFontSize) * 0.5f;
+		Cursor.m_StartX = Cursor.m_X;
+		Cursor.m_FontSize = InputFontSize;
 
 		// check if the visible text has to be moved
 		if(m_InputUpdate)
@@ -578,70 +683,269 @@ void CChat::OnRender()
 
 	y -= 8.0f;
 
+	// show all chat when typing
+	m_Show |= m_Mode != CHAT_NONE;
+
 	int64 Now = time_get();
-	float LineWidth = m_pClient->m_pScoreboard->Active() ? 90.0f : 200.0f;
-	float HeightLimit = m_pClient->m_pScoreboard->Active() ? 230.0f : m_Show ? 50.0f : 200.0f;
+	const int64 TimeFreq = time_freq();
+
+	// get scoreboard data
+	const bool IsScoreboardActive = m_pClient->m_pScoreboard->Active();
+	CUIRect ScoreboardRect = m_pClient->m_pScoreboard->GetScoreboardRect();
+	const CUIRect ScoreboardScreen = *UI()->Screen();
+	CUIRect ScoreboardRectFixed;
+	ScoreboardRectFixed.x = ScoreboardRect.x/ScoreboardScreen.w * Width;
+	ScoreboardRectFixed.y = ScoreboardRect.y/ScoreboardScreen.h * Height;
+	ScoreboardRectFixed.w = ScoreboardRect.w/ScoreboardScreen.w * Width;
+	ScoreboardRectFixed.h = ScoreboardRect.h/ScoreboardScreen.h * Height;
+
+	float LineWidth = 200.0f;
+	float HeightLimit = m_Show ? 90.0f : 200.0f;
+
+	if(IsScoreboardActive)
+	{
+		// calculate chat area (height gets a penalty as long lines are better to read)
+		float ReducedLineWidth = min(ScoreboardRectFixed.x - 5.0f - x, LineWidth);
+		float ReducedHeightLimit = max(ScoreboardRectFixed.y+ScoreboardRectFixed.h+5.0f, HeightLimit);
+		float Area1 = ReducedLineWidth * ((Height-HeightLimit) * 0.5f);
+		float Area2 = LineWidth * ((Height-ReducedHeightLimit) * 0.5f);
+
+		if(Area1 >= Area2)
+			LineWidth = ReducedLineWidth;
+		else
+			HeightLimit = ReducedHeightLimit;
+	}
+
 	float Begin = x;
 	float FontSize = 6.0f;
 	CTextCursor Cursor;
-	int OffsetType = m_pClient->m_pScoreboard->Active() ? 1 : 0;
+	int OffsetType = IsScoreboardActive ? 1 : 0;
+
+	// get the y offset (calculate it if we haven't done that yet)
 	for(int i = 0; i < MAX_LINES; i++)
 	{
 		int r = ((m_CurrentLine-i)+MAX_LINES)%MAX_LINES;
-		if(Now > m_aLines[r].m_Time+16*time_freq() && !m_Show)
-			break;
+		CLine& Line = m_aLines[r];
 
-		// get the y offset (calculate it if we haven't done that yet)
-		if(m_aLines[r].m_YOffset[OffsetType] < 0.0f)
+		if(m_aLines[r].m_aText[0] == 0) break;
+
+		if(Line.m_Size[OffsetType].y < 0.0f)
 		{
 			TextRender()->SetCursor(&Cursor, Begin, 0.0f, FontSize, 0);
 			Cursor.m_LineWidth = LineWidth;
-			TextRender()->TextEx(&Cursor, m_aLines[r].m_aName, -1);
-			TextRender()->TextEx(&Cursor, m_aLines[r].m_aText, -1);
-			m_aLines[r].m_YOffset[OffsetType] = Cursor.m_Y + Cursor.m_FontSize;
+
+			char aBuf[768] = {0};
+			if(Line.m_Mode == CHAT_TEAM)
+				str_format(aBuf, sizeof(aBuf), "[%s] ", Localize("Team"));
+			else if(Line.m_Mode == CHAT_WHISPER)
+				str_format(aBuf, sizeof(aBuf), "[%s] ", Localize("Whisper"));
+
+			if(Line.m_ClientID != -1)
+			{
+				Cursor.m_X += RenderTools()->GetClientIdRectSize(Cursor.m_FontSize);
+				str_append(aBuf, Line.m_aName, sizeof(aBuf));
+				str_append(aBuf, ": ", sizeof(aBuf));
+			}
+
+			str_append(aBuf, Line.m_aText, sizeof(aBuf));
+
+			TextRender()->TextEx(&Cursor, aBuf, -1);
+			// FIXME: sometimes an empty line will pop here when the cursor reaches the end of line
+			Line.m_Size[OffsetType].y = Cursor.m_LineCount * Cursor.m_FontSize;
+			Line.m_Size[OffsetType].x = Cursor.m_LineCount == 1 ? Cursor.m_X - Cursor.m_StartX : LineWidth;
 		}
-		y -= m_aLines[r].m_YOffset[OffsetType];
+	}
+
+	if(m_Show)
+	{
+		CUIRect Rect;
+		Rect.x = 0;
+		Rect.y = HeightLimit - 2.0f;
+		Rect.w = LineWidth + x;
+		Rect.h = Height - HeightLimit - 22.f;
+
+		const float LeftAlpha = 0.85f;
+		const float RightAlpha = 0.05f;
+		RenderTools()->DrawUIRect4(&Rect,
+								   vec4(0, 0, 0, LeftAlpha),
+								   vec4(0, 0, 0, RightAlpha),
+								   vec4(0, 0, 0, LeftAlpha),
+								   vec4(0, 0, 0, RightAlpha),
+								   CUI::CORNER_R, 3.0f);
+	}
+
+	for(int i = 0; i < MAX_LINES; i++)
+	{
+		int r = ((m_CurrentLine-i)+MAX_LINES)%MAX_LINES;
+		CLine& Line = m_aLines[r];
+
+		if(m_aLines[r].m_aText[0] == 0) break;
+
+		if(Now > Line.m_Time+16*TimeFreq && !m_Show)
+			break;
+
+		y -= Line.m_Size[OffsetType].y;
 
 		// cut off if msgs waste too much space
 		if(y < HeightLimit)
 			break;
 
-		float Blend = Now > m_aLines[r].m_Time+14*time_freq() && !m_Show ? 1.0f-(Now-m_aLines[r].m_Time-14*time_freq())/(2.0f*time_freq()) : 1.0f;
+		float Blend = Now > Line.m_Time+14*TimeFreq && !m_Show ? 1.0f-(Now-Line.m_Time-14*TimeFreq)/(2.0f*TimeFreq) : 1.0f;
+
+		const float HlTimeFull = 1.0f;
+		const float HlTimeFade = 1.0f;
+
+		float Delta = (Now - Line.m_Time) / (float)TimeFreq;
+		const float HighlightBlend = 1.0f - clamp(Delta - HlTimeFull, 0.0f, HlTimeFade) / HlTimeFade;
 
 		// reset the cursor
 		TextRender()->SetCursor(&Cursor, Begin, y, FontSize, TEXTFLAG_RENDER);
 		Cursor.m_LineWidth = LineWidth;
 
-		// render name
-		if(m_aLines[r].m_ClientID == -1)
-			TextRender()->TextColor(1.0f, 1.0f, 0.5f, Blend); // system
-		else if(m_aLines[r].m_Mode == CHAT_TEAM)
-			TextRender()->TextColor(0.45f, 0.9f, 0.45f, Blend); // team message
-		else if(m_aLines[r].m_NameColor == TEAM_RED)
-			TextRender()->TextColor(1.0f, 0.5f, 0.5f, Blend); // red
-		else if(m_aLines[r].m_NameColor == TEAM_BLUE)
-			TextRender()->TextColor(0.7f, 0.7f, 1.0f, Blend); // blue
-		else if(m_aLines[r].m_NameColor == TEAM_SPECTATORS)
-			TextRender()->TextColor(0.75f, 0.5f, 0.75f, Blend); // spectator
-		else
-			TextRender()->TextColor(0.8f, 0.8f, 0.8f, Blend);
+		const vec2 ShadowOffset(0.8f, 1.5f);
+		const vec4 ShadowWhisper(0.09f, 0.f, 0.26f, Blend * 0.9f);
+		const vec4 ShadowBlack(0, 0, 0, Blend * 0.9f);
+		vec4 ShadowColor = ShadowBlack;
 
-		TextRender()->TextEx(&Cursor, m_aLines[r].m_aName, -1);
+		if(Line.m_Mode == CHAT_WHISPER)
+			ShadowColor = ShadowWhisper;
+
+
+		const vec4 ColorSystem(1.0f, 1.0f, 0.5f, Blend);
+		const vec4 ColorWhisper(0.4f, 1.0f, 1.0f, Blend);
+		const vec4 ColorRed(1.0f, 0.5f, 0.5f, Blend);
+		const vec4 ColorBlue(0.7f, 0.7f, 1.0f, Blend);
+		const vec4 ColorSpec(0.75f, 0.5f, 0.75f, Blend);
+		const vec4 ColorAllPre(0.8f, 0.8f, 0.8f, Blend);
+		const vec4 ColorAllText(1.0f, 1.0f, 1.0f, Blend);
+		const vec4 ColorTeamPre(0.45f, 0.9f, 0.45f, Blend);
+		const vec4 ColorTeamText(0.6f, 1.0f, 0.6f, Blend);
+		const vec4 ColorHighlightBg(0.0f, 0.27f, 0.9f, 0.5f * HighlightBlend);
+		const vec4 ColorHighlightOutline(0.0f, 0.4f, 1.0f,
+			mix(Line.m_Mode == CHAT_TEAM ? 0.6f : 0.5f, 1.0f, HighlightBlend));
+
+		vec4 TextColor = ColorAllText;
+
+		if(Line.m_Highlighted && ColorHighlightBg.a > 0.001f)
+		{
+			CUIRect BgRect;
+			BgRect.x = Cursor.m_X;
+			BgRect.y = Cursor.m_Y + 2.0f;
+			BgRect.w = Line.m_Size[OffsetType].x - 2.0f;
+			BgRect.h = Line.m_Size[OffsetType].y;
+
+			vec4 LeftColor = ColorHighlightBg * ColorHighlightBg.a;
+			LeftColor.a = ColorHighlightBg.a;
+
+			vec4 RightColor = ColorHighlightBg;
+			RightColor *= 0.1f;
+
+			RenderTools()->DrawUIRect4(&BgRect,
+									   LeftColor,
+									   RightColor,
+									   LeftColor,
+									   RightColor,
+									   CUI::CORNER_R, 2.0f);
+		}
+
+		char aBuf[48];
+		if(Line.m_Mode == CHAT_WHISPER)
+		{
+			const float LineBaseY = TextRender()->TextGetLineBaseY(&Cursor);
+
+			const float qw = 10.0f;
+			const float qh = 5.0f;
+			const float qx = Cursor.m_X + 2.0f;
+			const float qy = LineBaseY - qh - 0.5f;
+
+			Graphics()->TextureSet(g_pData->m_aImages[IMAGE_CHATWHISPER].m_Id);
+			Graphics()->WrapClamp();
+
+			Graphics()->QuadsBegin();
+
+			// image orientation
+			const int LocalCID = m_pClient->m_LocalClientID;
+			if(Line.m_ClientID == LocalCID && Line.m_TargetID >= 0)
+				Graphics()->QuadsSetSubset(1, 0, 0, 1); // To
+			else if(Line.m_TargetID == LocalCID)
+				Graphics()->QuadsSetSubset(0, 0, 1, 1); // From
+			else
+				dbg_break();
+
+
+			// shadow pass
+			Graphics()->SetColor(ShadowWhisper.r*ShadowWhisper.a*Blend, ShadowWhisper.g*ShadowWhisper.a*Blend,
+								 ShadowWhisper.b*ShadowWhisper.a*Blend, ShadowWhisper.a*Blend);
+			IGraphics::CQuadItem Quad(qx + 0.2f, qy + 0.5f, qw, qh);
+			Graphics()->QuadsDrawTL(&Quad, 1);
+
+			// color pass
+			Graphics()->SetColor(ColorWhisper.r*Blend, ColorWhisper.g*Blend, ColorWhisper.b*Blend, Blend);
+			Quad = IGraphics::CQuadItem(qx, qy, qw, qh);
+			Graphics()->QuadsDrawTL(&Quad, 1);
+
+			Graphics()->QuadsEnd();
+			Graphics()->WrapNormal();
+			Cursor.m_X += 12.5f;
+		}
+
+		// render name
+		if(Line.m_ClientID == -1)
+			TextColor = ColorSystem;
+		else if(Line.m_Mode == CHAT_WHISPER)
+			TextColor = ColorWhisper;
+		else if(Line.m_Mode == CHAT_TEAM)
+			TextColor = ColorTeamPre;
+		else if(Line.m_NameColor == TEAM_RED)
+			TextColor = ColorRed;
+		else if(Line.m_NameColor == TEAM_BLUE)
+			TextColor = ColorBlue;
+		else if(Line.m_NameColor == TEAM_SPECTATORS)
+			TextColor = ColorSpec;
+		else
+			TextColor = ColorAllPre;
+
+		if(Line.m_ClientID != -1)
+		{
+			int NameCID = Line.m_ClientID;
+			if(Line.m_Mode == CHAT_WHISPER && Line.m_ClientID == m_pClient->m_LocalClientID && Line.m_TargetID >= 0)
+				NameCID = Line.m_TargetID;
+
+			vec4 IdTextColor = vec4(0.1f*Blend, 0.1f*Blend, 0.1f*Blend, 1.0f*Blend);
+			vec4 BgIdColor = TextColor;
+			BgIdColor.a = 0.5f*Blend;
+			RenderTools()->DrawClientID(TextRender(), &Cursor, NameCID, BgIdColor, IdTextColor);
+			str_format(aBuf, sizeof(aBuf), "%s: ", Line.m_aName);
+			TextRender()->TextShadowed(&Cursor, aBuf, -1, ShadowOffset, ShadowColor, TextColor);
+		}
 
 		// render line
-		if(m_aLines[r].m_ClientID == -1)
-			TextRender()->TextColor(1.0f, 1.0f, 0.5f, Blend); // system
-		else if(m_aLines[r].m_Highlighted)
-			TextRender()->TextColor(1.0f, 0.5f, 0.5f, Blend); // highlighted
-		else if(m_aLines[r].m_Mode == CHAT_TEAM)
-			TextRender()->TextColor(0.65f, 1.0f, 0.65f, Blend); // team message
+		if(Line.m_ClientID == -1)
+			TextColor = ColorSystem;
+		else if(Line.m_Mode == CHAT_WHISPER)
+			TextColor = ColorWhisper;
+		else if(Line.m_Mode == CHAT_TEAM)
+			TextColor = ColorTeamText;
 		else
-			TextRender()->TextColor(1.0f, 1.0f, 1.0f, Blend);
+			TextColor = ColorAllText;
 
-		TextRender()->TextEx(&Cursor, m_aLines[r].m_aText, -1);
+		if(Line.m_Highlighted)
+		{
+			TextRender()->TextColor(TextColor.r, TextColor.g, TextColor.b, TextColor.a);
+
+			TextRender()->TextOutlineColor(ColorHighlightOutline.r,
+										   ColorHighlightOutline.g,
+										   ColorHighlightOutline.b,
+										   ColorHighlightOutline.a);
+
+			TextRender()->TextEx(&Cursor, Line.m_aText, -1);
+		}
+		else
+			TextRender()->TextShadowed(&Cursor, Line.m_aText, -1, ShadowOffset, ShadowColor, TextColor);
 	}
 
 	TextRender()->TextColor(1.0f, 1.0f, 1.0f, 1.0f);
+	TextRender()->TextOutlineColor(0.0f, 0.0f, 0.0f, 0.3f);
 }
 
 void CChat::Say(int Mode, const char *pLine)
