@@ -13,6 +13,7 @@ public:
 	{
 		int m_CountryCode;
 		char m_aCountryCodeString[8];
+		bool m_Blocked;
 		IGraphics::CTextureHandle m_Texture;
 
 		bool operator<(const CCountryFlag &Other) { return str_comp(m_aCountryCodeString, Other.m_aCountryCodeString) < 0; }
@@ -22,8 +23,8 @@ public:
 
 	int Num() const;
 	const CCountryFlag *GetByCountryCode(int CountryCode) const;
-	const CCountryFlag *GetByIndex(int Index) const;
-	void Render(int CountryCode, const vec4 *pColor, float x, float y, float w, float h);
+	const CCountryFlag *GetByIndex(int Index, bool SkipBlocked = false) const;
+	void Render(int CountryCode, const vec4 *pColor, float x, float y, float w, float h, bool AllowBlocked=false);
 
 private:
 	enum
