@@ -24,6 +24,7 @@ class CMenusKeyBinder : public CComponent
 public:
 	bool m_TakeKey;
 	bool m_GotKey;
+	int m_Modifier;
 	IInput::CEvent m_Key;
 	CMenusKeyBinder();
 	virtual bool OnInput(IInput::CEvent Event);
@@ -47,13 +48,14 @@ private:
 
 
 	int DoButton_DemoPlayer(CButtonContainer *pBC, const char *pText, const CUIRect *pRect);
-	int DoButton_SpriteID(CButtonContainer *pBC, int ImageID, int SpriteID, const CUIRect *pRect, int Corners=CUI::CORNER_ALL, float r=5.0f, bool Fade=true);
+	int DoButton_SpriteID(CButtonContainer *pBC, int ImageID, int SpriteID, bool Checked, const CUIRect *pRect, int Corners=CUI::CORNER_ALL, float r=5.0f, bool Fade=true);
 	int DoButton_SpriteClean(int ImageID, int SpriteID, const CUIRect *pRect);
 	int DoButton_SpriteCleanID(const void *pID, int ImageID, int SpriteID, const CUIRect *pRect, bool Blend=true);
 	int DoButton_Toggle(const void *pID, int Checked, const CUIRect *pRect, bool Active);
 	int DoButton_Menu(CButtonContainer *pBC, const char *pText, int Checked, const CUIRect *pRect, const char *pImageName=0, int Corners=CUI::CORNER_ALL, float r=5.0f, float FontFactor=0.0f, vec4 ColorHot=vec4(1.0f, 1.0f, 1.0f, 0.75f), bool TextFade=true);
 	int DoButton_MenuTab(const void *pID, const char *pText, int Checked, const CUIRect *pRect, int Corners);
-	int DoButton_MenuTabTop(CButtonContainer *pBC, const char *pText, int Checked, const CUIRect *pRect, int Corners=CUI::CORNER_ALL, float r=5.0f, float FontFactor=0.0f);
+	int DoButton_MenuTabTop(CButtonContainer *pBC, const char *pText, int Checked, const CUIRect *pRect, float Alpha=1.0f, float FontAlpha=1.0f, int Corners=CUI::CORNER_ALL, float r=5.0f, float FontFactor=0.0f);
+	void DoButton_MenuTabTop_Dummy(const char *pText, int Checked, const CUIRect *pRect, float Alpha);
 	int DoButton_Customize(CButtonContainer *pBC, IGraphics::CTextureHandle Texture, int SpriteID, const CUIRect *pRect, float ImageRatio);
 
 	int DoButton_CheckBox_Common(const void *pID, const char *pText, const char *pBoxText, const CUIRect *pRect, bool Checked=false);
@@ -69,6 +71,7 @@ private:
 	*/
 
 	int DoIcon(int ImageId, int SpriteId, const CUIRect *pRect);
+	void DoIconColor(int ImageId, int SpriteId, const CUIRect *pRect, const vec4& Color);
 	int DoButton_GridHeader(const void *pID, const char *pText, int Checked, const CUIRect *pRect);
 	int DoButton_GridHeaderIcon(CButtonContainer *pBC, int ImageID, int SpriteID, const CUIRect *pRect, int Corners);
 
@@ -83,13 +86,14 @@ private:
 	void DoEditBoxOption(void *pID, char *pOption, int OptionLength, const CUIRect *pRect, const char *pStr, float VSplitVal, float *pOffset, bool Hidden=false);
 	void DoScrollbarOption(void *pID, int *pOption, const CUIRect *pRect, const char *pStr, float VSplitVal, int Min, int Max, bool infinite=false);
 	float DoDropdownMenu(void *pID, const CUIRect *pRect, const char *pStr, float HeaderHeight, FDropdownCallback pfnCallback);
+	float DoIndependentDropdownMenu(void *pID, const CUIRect *pRect, const char *pStr, float HeaderHeight, FDropdownCallback pfnCallback, bool* pActive);
 	void DoInfoBox(const CUIRect *pRect, const char *pLable, const char *pValue);
 	//static int ui_do_edit_box(void *id, const CUIRect *rect, char *str, unsigned str_size, float font_size, bool hidden=false);
 
 	float DoScrollbarV(const void *pID, const CUIRect *pRect, float Current);
 	float DoScrollbarH(const void *pID, const CUIRect *pRect, float Current);
 	void DoButton_KeySelect(CButtonContainer *pBC, const char *pText, int Checked, const CUIRect *pRect);
-	int DoKeyReader(CButtonContainer *pPC, const CUIRect *pRect, int Key);
+	int DoKeyReader(CButtonContainer *pPC, const CUIRect *pRect, int Key, int Modifier, int* NewModifier);
 
 	//static int ui_do_key_reader(void *id, const CUIRect *rect, int key);
 	void UiDoGetButtons(int Start, int Stop, CUIRect View, float ButtonHeight, float Spacing);
@@ -102,12 +106,32 @@ private:
 		CUIRect m_HitRect;
 	};
 
-	void UiDoListboxHeader(const CUIRect *pRect, const char *pTitle, float HeaderHeight, float Spacing);
-	void UiDoListboxStart(const void *pID, float RowHeight, const char *pBottomText, int NumItems,
-						int ItemsPerRow, int SelectedIndex, float ScrollValue, const CUIRect *pRect=0, bool Background=true);
-	CListboxItem UiDoListboxNextItem(const void *pID, bool Selected = false);
-	CListboxItem UiDoListboxNextRow();
-	int UiDoListboxEnd(float *pScrollValue, bool *pItemActivated);
+	struct CListBoxState
+	{
+		CUIRect m_ListBoxOriginalView;
+		CUIRect m_ListBoxView;
+		float m_ListBoxRowHeight;
+		int m_ListBoxItemIndex;
+		int m_ListBoxSelectedIndex;
+		int m_ListBoxNewSelected;
+		int m_ListBoxDoneEvents;
+		int m_ListBoxNumItems;
+		int m_ListBoxItemsPerRow;
+		float m_ListBoxScrollValue;
+		bool m_ListBoxItemActivated;
+
+		CListBoxState()
+		{
+			m_ListBoxScrollValue = 0;
+		}
+	};
+
+	void UiDoListboxHeader(CListBoxState* pState, const CUIRect *pRect, const char *pTitle, float HeaderHeight, float Spacing);
+	void UiDoListboxStart(CListBoxState* pState, const void *pID, float RowHeight, const char *pBottomText, int NumItems,
+						int ItemsPerRow, int SelectedIndex, const CUIRect *pRect=0, bool Background=true);
+	CListboxItem UiDoListboxNextItem(CListBoxState* pState, const void *pID, bool Selected = false, bool* pActive = NULL);
+	CListboxItem UiDoListboxNextRow(CListBoxState* pState);
+	int UiDoListboxEnd(CListBoxState* pState, bool *pItemActivated);
 
 	//static void demolist_listdir_callback(const char *name, int is_dir, void *user);
 	//static void demolist_list_callback(const CUIRect *rect, int index, void *user);
@@ -125,6 +149,7 @@ private:
 		POPUP_DELETE_DEMO,
 		POPUP_RENAME_DEMO,
 		POPUP_REMOVE_FRIEND,
+		POPUP_REMOVE_FILTER,
 		POPUP_SAVE_SKIN,
 		POPUP_DELETE_SKIN,
 		POPUP_SOUNDERROR,
@@ -141,35 +166,36 @@ private:
 		PAGE_CALLVOTE,
 		PAGE_INTERNET,
 		PAGE_LAN,
-		PAGE_FRIENDS,
 		PAGE_DEMOS,
 		PAGE_SETTINGS,
 		PAGE_SYSTEM,
 		PAGE_START,
 
-		PAGE_BROWSER_BROWSER=0,
-		PAGE_BROWSER_FRIENDS,
-		NUM_PAGE_BROWSER,
-
 		SETTINGS_GENERAL=0,
 		SETTINGS_PLAYER,
 		SETTINGS_TEE,
 		SETTINGS_CONTROLS,
 		SETTINGS_GRAPHICS,
 		SETTINGS_SOUND,
+
+		ACTLB_NONE=0,
+		ACTLB_LANG,
+		ACTLB_THEME,
 	};
 
 	int m_GamePage;
 	int m_Popup;
 	int m_ActivePage;
 	int m_MenuPage;
-	int m_BorwserPage;
 	bool m_MenuActive;
+	int m_SidebarTab;
+	bool m_SidebarActive;
+	bool m_ShowServerDetails;
 	bool m_UseMouseButtons;
 	vec2 m_MousePos;
 	vec2 m_PrevMousePos;
-	bool m_InfoMode;
 	bool m_PopupActive;
+	int m_ActiveListBox;
 
 	// images
 	struct CMenuImage
@@ -184,6 +210,45 @@ private:
 
 	const CMenuImage *FindMenuImage(const char* pName);
 
+	// themes
+	class CTheme
+	{
+	public:
+		CTheme() {}
+		CTheme(const char *n, bool HasDay, bool HasNight) : m_Name(n), m_HasDay(HasDay), m_HasNight(HasNight) {}
+
+		string m_Name;
+		bool m_HasDay;
+		bool m_HasNight;
+		IGraphics::CTextureHandle m_IconTexture;
+		bool operator<(const CTheme &Other) { return m_Name < Other.m_Name; }
+	};
+	sorted_array<CTheme> m_lThemes;
+
+	static int ThemeScan(const char *pName, int IsDir, int DirType, void *pUser);
+	static int ThemeIconScan(const char *pName, int IsDir, int DirType, void *pUser);
+
+	// gametype icons
+	class CGameIcon
+	{
+	public:
+		enum
+		{
+			GAMEICON_FULL=0,
+			GAMEICON_ON,
+			GAMEICON_OFF,
+		};
+		CGameIcon() {};
+		CGameIcon(const char *pName) : m_Name(pName) {}
+
+		string m_Name;
+		IGraphics::CTextureHandle m_IconTexture;
+	};
+	array<CGameIcon> m_lGameIcons;
+	IGraphics::CTextureHandle m_GameIconDefault;
+	void DoGameIcon(const char *pName, const CUIRect *pRect, int Type);
+	static int GameIconScan(const char *pName, int IsDir, int DirType, void *pUser);
+
 	int64 m_LastInput;
 
 	// loading
@@ -202,8 +267,11 @@ private:
 	static float ms_ButtonHeight;
 	static float ms_ListheaderHeight;
 	static float ms_FontmodHeight;
+	static float ms_BackgroundAlpha;
 
 	// for settings
+	bool m_NeedRestartPlayer;
+	bool m_NeedRestartTee;
 	bool m_NeedRestartGraphics;
 	bool m_NeedRestartSound;
 	int m_TeePartSelected;
@@ -216,7 +284,10 @@ private:
 	//
 	bool m_EscapePressed;
 	bool m_EnterPressed;
+	bool m_TabPressed;
 	bool m_DeletePressed;
+	bool m_UpArrowPressed;
+	bool m_DownArrowPressed;
 
 	// for map download popup
 	int64 m_DownloadLastCheckTime;
@@ -265,79 +336,42 @@ private:
 	class CFriendItem
 	{
 	public:
-		class CClanFriendItem
-		{
-		public:
-			CClanFriendItem()
-			{
-				m_lFriendInfos.clear();
-				m_lServerInfos.clear();
-			}
-
-			~CClanFriendItem()
-			{
-				m_lFriendInfos.clear();
-				m_lServerInfos.clear();
-			}
-
-			array<CFriendInfo> m_lFriendInfos;
-			array<const CServerInfo*> m_lServerInfos;
-		};
-
-		int m_NumFound;
-		const CFriendInfo *m_pFriendInfo;
 		const CServerInfo *m_pServerInfo;
-
-		CClanFriendItem m_ClanFriend;
+		char m_aName[MAX_NAME_LENGTH];
+		char m_aClan[MAX_CLAN_LENGTH];
+		int m_FriendState;
+		bool m_IsPlayer;
 
 		CFriendItem()
 		{
-			m_pFriendInfo = 0;
 			m_pServerInfo = 0;
 		}
 
-		bool IsClanFriend()
+		bool operator<(const CFriendItem &Other)
 		{
-			return m_pFriendInfo->m_aClan[0] && !m_pFriendInfo->m_aName[0];
+			if(m_aName[0] && !Other.m_aName[0])
+				return true;
+			if(!m_aName[0] && Other.m_aName[0])
+				return false;
+			int Result = str_comp_nocase(m_aName, Other.m_aName);
+			if(Result < 0 || (Result == 0 && str_comp_nocase(m_aClan, Other.m_aClan) < 0))
+				return true;
+			
+			return false;
 		}
-
-		void Reset()
-		{
-			m_NumFound = 0;
-			m_ClanFriend.m_lFriendInfos.clear();
-			m_ClanFriend.m_lServerInfos.clear();
-		}
-	};
-
-	struct CSelectedFriend
-	{
-		bool m_ClanFriend;
-		bool m_FakeFriend;
-		unsigned m_NameHash;
-		unsigned m_ClanHash;
 	};
 
 	enum
 	{
-		FRIENDS_SORT_TYPE = 0,
-		FRIENDS_SORT_SERVER,
-		FRIENDS_SORT_NAME,
-		FRIENDS_SORT_CLAN,
+		FRIEND_PLAYER_ON = 0,
+		FRIEND_CLAN_ON,
+		FRIEND_OFF,
+		NUM_FRIEND_TYPES
 	};
-
-	int *m_pFriendIndexes;
-	array<CFriendItem> m_lFriends;
-	int m_FriendlistSelectedIndex;
-	const CFriendInfo *m_pDeleteFriendInfo;
-	CSelectedFriend m_SelectedFriend;
-
-	bool SortCompareName(int Index1, int Index2) const;
-	bool SortCompareClan(int Index1, int Index2) const;
-	bool SortCompareServer(int Index1, int Index2) const;
-	bool SortCompareType(int Index1, int Index2) const;
+	sorted_array<CFriendItem> m_lFriendList[NUM_FRIEND_TYPES];
+	const CFriendItem *m_pDeleteFriend;
 
 	void FriendlistOnUpdate();
-	void SortFriends();
 
 	class CBrowserFilter
 	{
@@ -347,6 +381,10 @@ private:
 		int m_Filter;
 		class IServerBrowser *m_pServerBrowser;
 
+		static class CServerFilterInfo ms_FilterStandard;
+		static class CServerFilterInfo ms_FilterFavorites;
+		static class CServerFilterInfo ms_FilterAll;
+
 	public:
 
 		enum
@@ -358,9 +396,10 @@ private:
 		};
 		// buttons var
 		int m_SwitchButton;
+		int m_aButtonID[3];
 
 		CBrowserFilter() {}
-		CBrowserFilter(int Custom, const char* pName, IServerBrowser *pServerBrowser, int Filter, int Ping, int Country, const char* pGametype, const char* pServerAddress);
+		CBrowserFilter(int Custom, const char* pName, IServerBrowser *pServerBrowser);
 		void Switch();
 		bool Extended() const;
 		int Custom() const;
@@ -374,14 +413,18 @@ private:
 		const CServerInfo *SortedGet(int Index) const;
 		const void *ID(int Index) const;
 
-		void GetFilter(class CServerFilterInfo *pFilterInfo);
+		void Reset();
+		void GetFilter(class CServerFilterInfo *pFilterInfo) const;
 		void SetFilter(const class CServerFilterInfo *pFilterInfo);
 	};
 
 	array<CBrowserFilter> m_lFilters;
 
 	int m_SelectedFilter;
+	int m_RemoveFilterIndex;
 
+	void LoadFilters();
+	void SaveFilters();
 	void RemoveFilter(int FilterIndex);
 	void Move(bool Up, int Filter);
 
@@ -425,16 +468,7 @@ private:
 		COL_BROWSER_MAP,
 		COL_BROWSER_PLAYERS,
 		COL_BROWSER_PING,
-		COL_BROWSER_FAVORITE,
-		COL_BROWSER_INFO,
 		NUM_BROWSER_COLS,
-
-		COL_FRIEND_TYPE = 0,
-		COL_FRIEND_SERVER,
-		COL_FRIEND_NAME,
-		COL_FRIEND_CLAN,
-		COL_FRIEND_DELETE,
-		NUM_FRIEND_COLS,
 	};
 
 	struct CColumn
@@ -450,7 +484,6 @@ private:
 	};
 
 	static CColumn ms_aBrowserCols[NUM_BROWSER_COLS];
-	static CColumn ms_aFriendCols[NUM_FRIEND_COLS];
 
 	enum
 	{
@@ -465,9 +498,10 @@ private:
 		int m_WidthValue;
 		int m_HeightValue;
 	};
-	
+
 	CVideoFormat m_aVideoFormats[MAX_RESOLUTIONS];
-	sorted_array<CVideoMode> m_lFilteredVideoModes;
+	sorted_array<CVideoMode> m_lRecommendedVideoModes;
+	sorted_array<CVideoMode> m_lOtherVideoModes;
 	int m_NumVideoFormats;
 	int m_CurrentVideoFormat;
 	void UpdateVideoFormats();
@@ -500,16 +534,21 @@ private:
 	void RenderServerControlServer(CUIRect MainView);
 
 	// found in menus_browser.cpp
-	int m_ScrollOffset;
+	// int m_ScrollOffset;
 	void RenderServerbrowserServerList(CUIRect View);
+	void RenderServerbrowserSidebar(CUIRect View);
+	void RenderServerbrowserFriendTab(CUIRect View);
+	void RenderServerbrowserFilterTab(CUIRect View);
+	void RenderServerbrowserInfoTab(CUIRect View);
 	void RenderServerbrowserFriendList(CUIRect View);
+	void RenderDetailInfo(CUIRect View, const CServerInfo *pInfo);
+	void RenderDetailScoreboard(CUIRect View, const CServerInfo *pInfo, int RowCount, vec4 TextColor = vec4(1,1,1,1));
 	void RenderServerbrowserServerDetail(CUIRect View, const CServerInfo *pInfo);
-	void RenderServerbrowserFilters(CUIRect View);
 	//void RenderServerbrowserFriends(CUIRect View);
 	void RenderServerbrowserBottomBox(CUIRect View);
 	void RenderServerbrowserOverlay();
-	bool RenderFilterHeader(CUIRect View, int FilterIndex);
-	int DoBrowserEntry(const void *pID, CUIRect *pRect, const CServerInfo *pEntry, bool Selected);
+	void RenderFilterHeader(CUIRect View, int FilterIndex);
+	int DoBrowserEntry(const void *pID, CUIRect View, const CServerInfo *pEntry, const CBrowserFilter *pFilter, bool Selected);
 	void RenderServerbrowser(CUIRect MainView);
 	static void ConchainFriendlistUpdate(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
 	static void ConchainServerbrowserUpdate(IConsole::IResult *pResult, void *pUserData, IConsole::FCommandCallback pfnCallback, void *pCallbackUserData);
@@ -521,6 +560,7 @@ private:
 
 	// found in menus_settings.cpp
 	void RenderLanguageSelection(CUIRect MainView, bool Header=true);
+	void RenderThemeSelection(CUIRect MainView, bool Header=true);
 	void RenderHSLPicker(CUIRect Picker);
 	void RenderSkinSelection(CUIRect MainView);
 	void RenderSkinPartSelection(CUIRect MainView);
@@ -534,6 +574,9 @@ private:
 	void RenderSettingsSound(CUIRect MainView);
 	void RenderSettings(CUIRect MainView);
 
+	bool DoResolutionList(CUIRect* pRect, CListBoxState* pListBoxState,
+						  const sorted_array<CVideoMode>& lModes);
+
 	// found in menu_callback.cpp
 	static float RenderSettingsControlsMovement(CUIRect View, void *pUser);
 	static float RenderSettingsControlsWeapon(CUIRect View, void *pUser);
@@ -546,14 +589,19 @@ private:
 	void InvokePopupMenu(void *pID, int Flags, float X, float Y, float W, float H, int (*pfnFunc)(CMenus *pMenu, CUIRect Rect), void *pExtra=0);
 	void DoPopupMenu();
 
-	static int PopupFilter(CMenus *pMenus, CUIRect View);
-
 	IGraphics::CTextureHandle m_TextureBlob;
 
 	void ToggleMusic();
 
 	void SetMenuPage(int NewPage);
 public:
+	struct CSwitchTeamInfo
+	{
+		char m_aNotification[128];
+		bool m_AllowSpec;
+		int m_TimeLeft;
+	};
+	void GetSwitchTeamInfo(CSwitchTeamInfo *pInfo);
 	void RenderBackground();
 
 	void UseMouseButtons(bool Use) { m_UseMouseButtons = Use; }
@@ -569,6 +617,7 @@ public:
 	virtual void OnInit();
 
 	virtual void OnConsoleInit();
+	virtual void OnShutdown();
 	virtual void OnStateChange(int NewState, int OldState);
 	virtual void OnReset();
 	virtual void OnRender();
